@@ -2,7 +2,6 @@
 #   Pyrex - Types
 #
 
-import string
 import Naming
 
 class BaseType:
@@ -37,6 +36,7 @@ class PyrexType(BaseType):
     #  is_enum               boolean     Is a C enum type
     #  is_typedef            boolean     Is a typedef type
     #  is_string             boolean     Is a C char * type
+    #  is_unicode            boolean     Is a UTF-8 encoded C char * type
     #  is_returncode         boolean     Is used only to signal exceptions
     #  is_error              boolean     Is the dummy error type
     #  has_attributes        boolean     Has C dot-selectable attributes
@@ -71,6 +71,7 @@ class PyrexType(BaseType):
         
     is_pyobject = 0
     is_extension_type = 0
+    is_builtin_type = 0
     is_numeric = 0
     is_int = 0
     is_float = 0
@@ -83,6 +84,7 @@ class PyrexType(BaseType):
     is_enum = 0
     is_typedef = 0
     is_string = 0
+    is_unicode = 0
     is_returncode = 0
     is_error = 0
     has_attributes = 0
@@ -100,7 +102,7 @@ class PyrexType(BaseType):
         return str(value)
     
     def __str__(self):
-        return string.strip(self.declaration_code("", for_display = 1))
+        return self.declaration_code("", for_display = 1).strip()
     
     def same_as(self, other_type, **kwds):
         return self.same_as_resolved_type(other_type.resolve(), **kwds)
@@ -208,6 +210,48 @@ class PyObjectType(PyrexType):
             return "%s *%s" % (public_decl("PyObject", dll_linkage), entity_code)
 
 
+class BuiltinObjectType(PyObjectType):
+
+    is_builtin_type = 1
+    has_attributes = 1
+    base_type = None
+    module_name = '__builtin__'
+
+    def __init__(self, name, cname):
+        self.name = name
+        self.cname = cname
+        self.typeptr_cname = "&" + cname
+                                 
+    def set_scope(self, scope):
+        self.scope = scope
+        if scope:
+            scope.parent_type = self
+        
+    def __str__(self):
+        return "%s object" % self.name
+    
+    def __repr__(self):
+        return "<%s>"% self.cname
+        
+    def assignable_from(self, src_type):
+        if isinstance(src_type, BuiltinObjectType):
+            return src_type.name == self.name
+        else:
+            return not src_type.is_extension_type
+            
+    def typeobj_is_available(self):
+        return True
+        
+    def attributes_known(self):
+        return True
+        
+    def subtype_of(self, type):
+        return type.is_pyobject and self.assignable_from(type)
+        
+    def type_test_code(self, arg):
+        return 'likely(Py%s_CheckExact(%s)) || (%s) == Py_None || (PyErr_Format(PyExc_TypeError, "Expected %s, got %%s", %s->ob_type->tp_name), 0)' % (self.name[0].upper() + self.name[1:], arg, arg, self.name, arg)
+
+
 class PyExtensionType(PyObjectType):
     #
     #  A Python extension type.
@@ -279,6 +323,9 @@ class PyExtensionType(PyObjectType):
             else:
                 return "%s *%s" % (base,  entity_code)
 
+    def type_test_code(self, py_arg):
+        return "__Pyx_TypeTest(%s, %s)" % (py_arg, self.typeptr_cname)
+    
     def attributes_known(self):
         return self.scope is not None
     
@@ -607,7 +654,7 @@ class CFuncType(CType):
         return "<CFuncType %s %s[%s]>" % (
             repr(self.return_type),
             self.calling_convention_prefix(),
-            string.join(arg_reprs, ","))
+            ",".join(arg_reprs))
     
     def calling_convention_prefix(self):
         cc = self.calling_convention
@@ -738,7 +785,7 @@ class CFuncType(CType):
             arg_decl_list.append(self.op_arg_struct.declaration_code(Naming.optional_args_cname))
         if self.has_varargs:
             arg_decl_list.append("...")
-        arg_decl_code = string.join(arg_decl_list, ", ")
+        arg_decl_code = ", ".join(arg_decl_list)
         if not arg_decl_code and not pyrex:
             arg_decl_code = "void"
         exc_clause = ""
@@ -875,19 +922,50 @@ class CEnumType(CType):
             return self.base_declaration_code(public_decl(base, dll_linkage), entity_code)
 
 
+def _escape_byte_string(s):
+    s = s.replace('\0', r'\x00')
+    try:
+        s.decode("ASCII")
+        return s
+    except UnicodeDecodeError:
+        pass
+    l = []
+    append = l.append
+    for c in s:
+        o = ord(c)
+        if o >= 128:
+            append('\\x%X' % o)
+        else:
+            append(c)
+    return ''.join(l)
+
 class CStringType:
     #  Mixin class for C string types.
 
     is_string = 1
+    is_unicode = 0
     
     to_py_function = "PyString_FromString"
     from_py_function = "PyString_AsString"
     exception_value = "NULL"
 
     def literal_code(self, value):
-        if isinstance(value, unicode):
-            value = value.encode("UTF-8")
-        return '"%s"' % value
+        assert isinstance(value, str)
+        return '"%s"' % _escape_byte_string(value)
+
+
+class CUTF8StringType:
+    #  Mixin class for C unicode types.
+
+    is_string = 1
+    is_unicode = 1
+    
+    to_py_function = "PyUnicode_DecodeUTF8"
+    exception_value = "NULL"
+
+    def literal_code(self, value):
+        assert isinstance(value, str)
+        return '"%s"' % _escape_byte_string(value)
 
 
 class CCharArrayType(CStringType, CArrayType):
@@ -898,6 +976,16 @@ class CCharArrayType(CStringType, CArrayType):
     
     def __init__(self, size):
         CArrayType.__init__(self, c_char_type, size)
+
+
+class CUTF8CharArrayType(CUTF8StringType, CArrayType):
+    #  C 'char []' type.
+    
+    parsetuple_format = "s"
+    pymemberdef_typecode = "T_STRING_INPLACE"
+    
+    def __init__(self, size):
+        CArrayType.__init__(self, c_char_type, size)
     
 
 class CCharPtrType(CStringType, CPtrType):
@@ -910,6 +998,16 @@ class CCharPtrType(CStringType, CPtrType):
         CPtrType.__init__(self, c_char_type)
 
 
+class CUTF8CharPtrType(CUTF8StringType, CPtrType):
+    # C 'char *' type, encoded in UTF-8.
+    
+    parsetuple_format = "s"
+    pymemberdef_typecode = "T_STRING"
+    
+    def __init__(self):
+        CPtrType.__init__(self, c_char_type)
+
+
 class ErrorType(PyrexType):
     # Used to prevent propagation of error messages.
     
@@ -974,7 +1072,9 @@ c_longdouble_type =  CFloatType(8)
 
 c_null_ptr_type =     CNullPtrType(c_void_type)
 c_char_array_type =   CCharArrayType(None)
+c_utf8_char_array_type =   CUTF8CharArrayType(None)
 c_char_ptr_type =     CCharPtrType()
+c_utf8_char_ptr_type =     CUTF8CharPtrType()
 c_char_ptr_ptr_type = CPtrType(c_char_ptr_type)
 c_int_ptr_type =      CPtrType(c_int_type)
 
